# This file is required to create a python library

from pathlib import Path

from setuptools import find_packages, setup

p = Path("README.md")
long_descr = p.read_text()

setup(
    name='echr_extractor',
    packages=find_packages(include=['echr_extractor']),
    version='1.0.39',
    description='Library for extracting ECHR data',
    author='LawTech Lab',
    license='MIT',
    install_requires=["requests~=2.26.0", "pandas", "beautifulsoup4~=4.9.3", "dateparser", "tqdm"],
    author_email='a.gade@student.maastrichtuniversity.nl',
    keywords=['echr', 'extractor', 'european', 'convention', 'human', 'rights', 'european convention', 'human rights',
              'european convention on human rights'],
    long_description=long_descr,
    long_description_content_type='text/markdown',
    project_urls={
        "Bug Tracker": "https://github.com/maastrichtlawtech/extraction_libraries",
        "Build Source": "https://github.com/maastrichtlawtech/extraction_libraries",
    },
)
